#define port GPIO
#define tris TRISIO

volatile char Count_500 = 0, Flag = 0;  // 500us counter and null flag
volatile unsigned int Count_10000 = 0;  // 0.01 s counter










//========================== calibration const. ================================
void calibr(void) org 0x3FE {
  asm nop;
 }
//==============================================================================





// ===================== calling EEPROM_Write with int off =====================
void EEprom_Write_(char adr, char x) {
 EEPROM_Write(adr, x);
 while (EECON1.WR);   // wait to write complete
 }
//==============================================================================






// ===================== Init. EEPROM ==========================================
// At adress 0x00-0x01 - normal current of lamp  (0 - small bit)
// At adress 0x02   -  regime of delay   (0... 5)
// =============================================================================
void Init_EE(void) {
  if (EEprom_Read(2) == 0xFF) {
     EEprom_Write(2, 0);
     while (EECON1.WR);   // wait to write complete
     }
  }
// =============================================================================









// ===================== Corrent overlimit =====================================
void Error(void){
 port = 0x20;   // lamp --> off, led to on
 while(1);
 
}
// =============================================================================







// ===================== ADC reading (10 bit) ===================================
unsigned int read_ADC(){
  unsigned int x;

  ADCON0.GO = 1;          // start
  while (ADCON0.GO);      // wait to convertions end

  x = (ADRESH << 8) + ADRESL;
  if(x > 1000) Error();

  return x;    //

 }
//==============================================================================











// ======================== Main ===============================================
void main() {
 char Delay_1, Delay_2, Delay_x,  i, Counter;
 unsigned int Current_0, Current_x;
 char K[] = {9, 11, 13, 15, 17, 21, 25, 31, 41, 59};//carrent to rms coefficient
 
 calibr();                 // -------------------------------
 asm {                     // osc. calibration
   bsf STATUS, RP0;
   movlw 0x80;             // middle if constante not saved
   call 0x3FF;
   movwf OSCCAL ;
   bcf STATUS, RP0;
  }

 
 Init_EE();  // EEprom init.
 
 Current_0 =(EEprom_Read(1) << 8)+ EEprom_Read(0);//read the end current of lamp
 
 Delay_1 = EEprom_Read(2);            // read  old delay regime
 Delay_2 = Delay_1 + 1;               //
 if(Delay_2 > 5) Delay_2 = 0;         // 0, 1, ... 5, 0...
 EEprom_Write(2, Delay_2);            // write new delay regime
 Delay_2 = Delay_1 << 3;      // (*8)
 if (Delay_2 == 0) Delay_2++; // = (1, 8, 16, 24, 32, 40)

 tris = 0b00001110;
 port = 0;
 
 CMCON = 7;
 
 ANSEL = 0x12;      // ---------ADC configure  ----------------- AN1 on, Tosc/8
 ADCON0 = 0x85;     // ---------------- ref = Vdd, right jastify, channel 1, on

 OPTION_REG = 0b11000000; //  pos. front int, prescaler - 2
 INTCON = 0b10110000;     // interrupts on, by tmr0, by intf
 
 i = 0;
 Counter = 18;
 while(1){
    Delay_x = Delay_2;
    while (Count_10000 < 500) {        // loop to 5s max.
       while(Flag.F0 == 0);  // wait the null crossing
       Flag.F0 = 0;  // clear null Flag
       
       while(Counter != Count_500);  // wait to current delay
       port = 1;    // switch on the thyristor
       if((--Delay_x == 0)||((char)(Delay_x + 1) == Delay_2)) Current_x = read_ADC();
                                                  //  pulse current  mesure
                                                  // at first and last time
       while(TMR0 < 100);      // wait 0.2 ms in on state
       port = 0;    // switch off the control pulse
 
       if(Delay_x != 0) continue;
       Current_x = K[i] * Current_x;
       if(Current_x < Current_0) break; // repeat if current is more than normal
      }
      
    i++;
    if(Counter > 14) Counter--;            // to next value
       else if(Counter > 4) Counter -= 2;  //
               else break;                 //
               
   }


   while(1){           // ----------------- the normal work --------------------

       if((Count_10000 > 600)&&(Flag.F7==0)&&(Count_500 == 10)) {
         Flag.F7 = 1;         // only one time  after  6 s

         Current_x = 35 * read_ADC(); // normal rms  current
         if(Current_x > 10500) Error();   // > 200 W

         EEprom_Write(2, Delay_1);            // restore old delay regime
         Flag.F6 = 1;  // set 1-st EEPROM write flag
        }

       if(Flag.F0 == 0) continue;  // wait the null crossing for go
       Flag.F0 = 0;  // clear null Flag

       port = 1;    // switch on the thyristor
       while(TMR0 < 100);      // wait 0.2 ms in on state
       port = 0;    // switch off the control pulse

           // ---- save the rms current in normal work  (without any delay)-----
       if((Flag.F6==1)&& (EECON1.WR==0)) {
          EEprom_Write(0, Current_x );
          Flag.F6 = 0; // clear 1-st EEPROM write Flag
          Flag.F5 = 1; // set 2-nd EEPROM write flag
          }
       if((Flag.F5==1)&& (EECON1.WR==0)) {
          EEprom_Write(1, Current_x >> 8 );
          Flag.F5 = 0; // clear 2-nd EEPROM write flag
          } // -----------------------------------------------------------------

       while (Count_500 < 1);
       if(read_ADC() <= 1) {
          port = 1;               //  if current less than 10 mA
          while(TMR0 < 100);      //  to 0.5 ms after nullcrossing
          port = 0;               //  repeat the switching on

         }

     }

 }
 

//========================= end of main ========================================







//  *************************  INTERRUPTS  *************************************

void interrupt(){
   if (INTCON.TMR0IF == 1){
   
      asm {nop; nop;}    //
      TMR0 += 7;         // interval 250*2 = 500us
      
      Count_500++;    // increment the 500us counter
   
      INTCON.TMR0IF = 0; // clear int. flag
    }


   if (INTCON.INTF == 1){
   
      TMR0 = 50;           // clear TMR0 + delay of zero cross detection
      INTCON.TMR0IF = 0; // clear TMR0 int. flag
      Flag.F0 = 1;      // set null flag
      Count_500 = 0;    // clear 500us counter
      Count_10000++;
      INTCON.INTF = 0;  // clear int. flag
    }


 }     //         End of interrupts

 //*******************************************************************************